/*---------------------------------------------------------------------------
**
**	PLXCFG.C
**
**
**	Copyright(c) 1999-2000, Vital Computer Solutions, All Rights Reserved
**	Richard A. Reynolds
**
**	Copyright(c) 2000-2004, ProSoft Technology, All Rights Reserved
**	Ken D. Hopwood
**
**	Copyright(c) 2002-2003, ProSoft Technology, All Rights Reserved
**	Sukhum Amorntrakul (Joe)
**
**	Incorporates code from 01/2000 to 06/2000
**
**	MODIFIED :
**
**		12/14/2000 - 03/05/2001 -- Created example code from
**									existing works.
**		03/21/2001 -- added comments
**
**
**	Title:	This file contains the backplane example interface functions
**			for use with the ADM API.
**
**  Abstract:
**
**
**	This file contains the configuration validation logic.  The user
**	may modify the parameter checking to match the configuration needs
**	of the application.
**
**  Environment:	ProLinx
**						GS-DOS
**						Borland C/C++ Compiler v5.02 (16-bit DOS)
**						Digital Mars C/C++ Compiler v8.49 (16-bit DOS)
**
**
** ProSoft Technology, Inc. grants you a non-exclusive license to use, modify
** and re-distribute this program provided that this copyright notice and
** license appear on all copies of the software.
**
** Software is provided "AS IS," without a warranty of any kind. ALL EXPRESS OR
** IMPLIED REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
** MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
** HEREBY EXCLUDED. THE ENTIRE RISK ARISING OUT OF USING THE SOFTWARE IS ASSUMED
** BY THE LICENSEE.
**
** PROSOFT TECHNOLOGY, INC. AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY
** DAMAGES SUFFERED BY LICENSEE OR ANY THIRD PARTY AS A RESULT OF USING OR
** DISTRIBUTING SOFTWARE.  IN NO EVENT WILL PROSOFT TECHNOLOGY, INC. OR ITS
** LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
** INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
** CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
** OR INABILITY TO USE SOFTWARE, EVEN IF PROSOFT TECHNOLOGY, INC. HAS BEEN
** ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
**
**---------------------------------------------------------------------------
*/

/*---------------------------------------------------------------------------
**	Include files
**---------------------------------------------------------------------------
*/

#include "main_plx.h"		/* primary application header file */
#include <string.h>
#include <stdio.h>


/*---------------------------------------------------------------------------
**	CONSTANT DEFINITIONS
**---------------------------------------------------------------------------
*/



/*---------------------------------------------------------------------------
**	Function prototypes
**---------------------------------------------------------------------------
*/

static void InitCfg(void);
int ReadCfg(void);


/*---------------------------------------------------------------------------
**	Variable definitions
**---------------------------------------------------------------------------
*/

extern ADM_BT_DATA	bt_data;
extern ADM_PORT ports[];
extern ADM_MODULE module;
extern ADMHANDLE adm_handle;				/* handle to ADM API */
extern ADM_INTERFACE interface;


/*---------------------------------------------------------------------------
** LOCAL MODULE DATA
**---------------------------------------------------------------------------
*/

static unsigned short MinDelay[] =
	{
		350,					/* #define     BAUD_110        0 */
		256,					/* #define     BAUD_150        1 */
		128,					/* #define     BAUD_300        2 */
		64, 					/* #define     BAUD_600        3 */
		32, 					/* #define     BAUD_1200       4 */
		16, 					/* #define     BAUD_2400       5 */
		8,  					/* #define     BAUD_4800       6 */
		4,  					/* #define     BAUD_9600       7 */
		2,  					/* #define     BAUD_19200      8 */
		2,  					/* #define     BAUD_28800      9 */
		1,  					/* #define     BAUD_38400      10 */
		0,  					/* #define     BAUD_57600      11 */
		0,  					/* #define     BAUD_115200     12 */
		0,0,0,0
	};


/*---------------------------------------------------------------------------
**	Functions
**---------------------------------------------------------------------------
*/



/*---------------------------------------------------------------------------
**	This function is used to gather the configuration information
**	sent from the flash into the modules configuration.
**---------------------------------------------------------------------------
*/

int ReadCfg(void)
{
	char huge * cptr;
	char huge * mptr;
	char huge * tptr;
	char section[40];
	int i;

   if(ADM_PLX_ReadConfiguration(adm_handle, &tptr) == 0)	//if no configuration data, return
	{
		printf("ERROR: No configuration return\n");
		return (1);
	}
	InitCfg();

	if((tptr = ADM_PLX_FindSection(adm_handle, "[Module]", tptr)) != NULL)
	{
		mptr = tptr;
		cptr = (char*)ADM_RAM_GetString(adm_handle, mptr, "Module Name");
		if(cptr == NULL)
      {
			strcpy(module.name, "No Module Name");
			printf("Can't find Module Name\n");
      }
		else
		{
			if(strlen((char *)cptr) > 80)
				*(cptr+80) = 0;
			strcpy(module.name, (char *)cptr);
			if(module.name[strlen(module.name)-1] < 32)
				module.name[strlen(module.name)-1] = 0;
		}

		module.err_offset = ADM_RAM_GetInt(adm_handle, mptr, "Error/Status Block Pointer");
		if(module.err_offset < 0 || module.err_offset > module.max_regs-61)
		{
			module.err_offset = -1;
			module.err_freq   =  0;
		}
		else
		{
			module.err_freq   =  500;
		}

		module.err_freq = ADM_RAM_GetInt(adm_handle, mptr, "Error/Status Frequency");
	}

	for(i = 0; i < MAX_APP_PORTS; i++)
	{
		sprintf(section, "[Port %d]", i);

		if((tptr = ADM_PLX_FindSection(adm_handle, section, tptr)) != NULL)
		{
			mptr = tptr;
			*cptr = ADM_RAM_GetChar(adm_handle, mptr, "Enable");

			if(*cptr == 'Y')
	      {
				ports[i].enabled = 'Y';
			}
	      else
	      {
				ports[i].enabled = 'N';
			}

			if(i) //only for upper ports
			{
				ports[i].interface = ADM_RAM_GetInt(adm_handle, mptr, "RS Interface");
				if(ports[i].interface < 0 || ports[i].interface > 2)
				{
					ports[i].interface = 1;
				}
				else
            {
					ports[i].interface++;
            }
			}

			// check port parameters only if enabled
			if(ports[i].enabled == 'Y')
			{
				ports[i].baud = ADM_RAM_GetInt(adm_handle, mptr, "Baud Rate");

				switch(ports[i].baud)
				{
					case 110:
						ports[i].baud = BAUD_110;
						break;

					case 150:
						ports[i].baud = BAUD_150;
						break;

					case 300:
						ports[i].baud = BAUD_300;
						break;

					case 600:
						ports[i].baud = BAUD_600;
						break;

					case 12:
					case 1200:
						ports[i].baud = BAUD_1200;
						break;

					case 24:
					case 2400:
						ports[i].baud = BAUD_2400;
						break;

					case 48:
					case 4800:
						ports[i].baud = BAUD_4800;
						break;

					case 96:
					case 9600:
					default :
						ports[i].baud = BAUD_9600;
						break;

					case 19:
					case 192:
					case 19200:
						ports[i].baud = BAUD_19200;
						break;

					case 28:
					case 288:
					case 28800:
						ports[i].baud = BAUD_28800;
						break;

					case 38:
					case 384:
					case 38400L:
						ports[i].baud = BAUD_38400;
						break;

					case 57:
					case 576:
						ports[i].baud = BAUD_57600;
						break;

					case 115:
					case 1152:
						ports[i].baud = BAUD_115200;
						break;
				}

				*cptr = ADM_RAM_GetChar(adm_handle, mptr, "Parity");
				switch(*cptr)
				{
					case 'O':
					case 'o':
						ports[i].parity = PARITY_ODD;
						break;

					case 'E':
					case 'e':
						ports[i].parity = PARITY_EVEN;
						break;

					case 'M':
					case 'm':
						ports[i].parity = PARITY_MARK;
						break;

					case 'S':
					case 's':
						ports[i].parity = PARITY_SPACE;
						break;

					case 'N':
					case 'n':
					default :
						ports[i].parity = PARITY_NONE;
						break;
				}

				ports[i].databits      = ADM_RAM_GetInt(adm_handle, mptr, "Data Bits");
				switch(ports[i].databits)
				{
					case 5:
						ports[i].databits = WORDLEN5;
						break;

					case 6:
						ports[i].databits = WORDLEN6;
						break;

					case 7:
						ports[i].databits = WORDLEN7;
						break;

					case 8:
					default :
						ports[i].databits = WORDLEN8;
						break;
				}

				ports[i].stopbits      = ADM_RAM_GetInt(adm_handle, mptr, "Stop Bits");
				switch(ports[i].stopbits)
				{
					case 1:
					default :
						ports[i].stopbits = STOPBITS1;
						break;

					case 2:
						ports[i].stopbits = STOPBITS2;
						break;
				}

				ports[i].RTS_On       = ADM_RAM_GetInt(adm_handle, mptr, "RTS On");

				ports[i].RTS_Off      = ADM_RAM_GetInt(adm_handle, mptr, "RTS Off");

				ports[i].MinDelay     = ADM_RAM_GetInt(adm_handle, mptr, "Minimum Command Delay");
				if(ports[i].MinDelay == 0)
					ports[i].MinDelay = MinDelay[ports[i].baud];

				*cptr = ADM_RAM_GetChar(adm_handle, mptr, "Use CTS Line");
				if(*cptr == 'Y')
            {
					ports[i].CTS = 'Y';
				}
            else
            {
					ports[i].CTS = 'N';
				}
			}
		}
      else
      {
			printf("Can't find section %s\n", section);
      }
	}
	return(0);
}



/*---------------------------------------------------------------------------
**	This function is used to initialize the configuration information
**	in the module.
**---------------------------------------------------------------------------
*/

static void InitCfg(void)
{
	int i;

	strcpy(module.name, "NOT CONFIGURED");
	module.max_regs      = ADM_MAX_DB_REGS;
	module.err_offset    = -1;
	module.err_freq      = 0;

   for(i=0;i<MAX_APP_PORTS;i++)
   {
		ports[i].enabled	= 'Y';
		ports[i].baud		= BAUD_9600;
		ports[i].parity 	= PARITY_NONE;
		ports[i].databits	= WORDLEN8;
		ports[i].stopbits	= STOPBITS1;
		ports[i].RTS_On 	= 0;
		ports[i].RTS_Off	= 0;
		ports[i].MinDelay	= 0;
		ports[i].CTS		= 'N';
   }
}


